% Solve 1-D Rosenau equation on [-L,L] with 
% Pseudo-Spectral Fictitious point (FP-PS) method
%
% Equation
% u_t + 0.5*u_xxxxt = g(u)_x
% 
% with boundary conditions
% u(-L,t) = sech(-L-t)
% u(+L,t) = sech(+L-t)
% u_x(-L,t) = sech(t+L)tanh(t+L)
% u_x(+L,t) = sech(t-L)tanh(t-L)
%
% Initial condition: u(x,t) = sech(x-t)
% g(u) = 10u^3 - 12u^5 - 3u/2
% g(u)_x = (-3/2)u_x - 60u^4u_x + 30u^2u_x

% REMARK:
% Some functions of Chebfun package is required
% Download Chebfun and add it to the path
% http://www.chebfun.org/

function [S,x,t] = FP_PS(N,L,T)
% N : The number of the cluster node points
% L : [-L,L] is the domain
% T : Final time

% Exact solution and derivatives for test case
u = @(x,t) sech(x-t);
ux = @(x,t) sech(t-x).*tanh(t-x);
ut = @(x,t) -sech(t-x).*tanh(t-x);
uxt = @(x,t) sech(t - x).^3 - sech(t - x).*tanh(t - x).^2;

x = chebpts(N); % Chebyshev points
% Differentiation matrices
D = (1/L)*diffmat(N,1); D4 = (L^(-4))*diffmat(N,4);
% Map points such that x(2) and x(N-1) are dom(1) and dom(2) respectively
% and x(1) and x(N) become the fictitious points.
linmap = @(x,x1,x2,y1,y2) (y2-y1)*(x-x1)/(x2-x1) + y1;
x = linmap(x,x(2),x(N-1),-L,L);
% Update differentiation matrices due to the chain rule
const = (x(N-1)-x(2))/(x(end)-x(1));
Dx = const*D; D4x = const^4*D4;

Bf = Dx([2 N-1],[1 N]); Bd = Dx([2 N-1],3:N-2); Bb = Dx([2 N-1],[2,N-1]);
% Modify differentiation matrices
Dxd = Dx(3:N-2,3:N-2) - (Dx(3:N-2,[1 N])/Bf)*Bd;
Dxb = [Dx(3:N-2,[2 N-1]) - (Dx(3:N-2,[1 N])/Bf)*Bb  Dx(3:N-2,[1 N])/Bf];
D4xd = D4x(3:N-2,3:N-2) - (D4x(3:N-2,[1 N])/Bf)*Bd;
D4xb = [D4x(3:N-2,[2 N-1]) - (D4x(3:N-2,[1 N])/Bf)*Bb  D4x(3:N-2,[1 N])/Bf];
% Initial condition
S0 = sech(x(3:N-2)); opt = odeset('RelTol',1e-9);
% Solve the ODE for the approximate solution S
fun = @(t,S) odefun(t,S,x,N,Dxd,Dxb,D4xd,D4xb,u,ux,ut,uxt);
[t,S] = ode15s(fun,[0 T],S0,opt);
% Plot the solution for all times
figure(1),clf,plot(x(3:N-2),S)
% Plot the error at the final time
E = abs(S(end,:)-u(x(3:N-2),T)');
figure(2),clf,plot(x(3:N-2),E)
% figure(3);
% for i=1:length(t)
% 	subplot(2,1,1)
% 	plot(x(3:N-2),S(i,:),'o',x(3:N-2),u(x(3:N-2),t(i)),'--r');
% 	xlabel('x'); ylabel('s(x,t)'); xlim([-L L]); ylim([0 1])
% 	drawnow;
% 	subplot(2,1,2)
% 	semilogy(t(i),max(E),'*b');
% 	xlabel('t'); ylabel('error'); ylim([1e-6 1e-5])
% 	set(gca,'ytick',10.^(-7:0)); hold on;
% end
function Sprime = odefun(t,S,x,N,Dxd,Dxb,D4xd,D4xb,u,ux,ut,uxt)
Fx = [u(x([2 N-1]),t); ux(x([2 N-1]),t)];
F4xt = [ut(x([2 N-1]),t); uxt(x([2 N-1]),t)];
Gu = diag(-1.5 - 60*S.^4 + 30*S.^2);
Sprime = (eye(N-4) + 0.5*D4xd)\(Gu*Dxd*S + Gu*Dxb*Fx - 0.5*D4xb*F4xt);


