%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve 2-D Rosenau equation on [-L,L]x[-L,L] with RBF
% collocation and Fictitious point RBF method.
% u_t + \Delta^2{u} = \nabla g(u) , g(u) = u^2 + u^3.
% u(x,y,0) = sech(x + y)
% u(x,y,t) = f(x,y,t),    x \in \partial\Omega ,
% ux(x,y,t) = Df(x,y,t),  x \in \partial\Omega ,
% where f(x,y,t) = sech(x + y - t) 
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t,S] = FP_RBF_2D(n,L,T,ep)
% n : The number of node points in each direction
% L : [-L,L]x[-L,L] is the domain
% T : Final time
% ep: Shape parameter. 
% For the case "FP_RBF_2D(n,L,T)", the shape paramet is
% ep = 0.1/h;

gu = @(u) 2*u + 3*u.^2; % Non-linear function g_u
uini = @(x,y) sech(x+y); % Initial condition
ub = @(x,y,t) sech(x+y-t); % Boundary conditions
ux = @(x,y,t) -sech(x+y-t).*tanh(x+y-t);
ut = @(x,y,t) sech(x+y-t).*tanh(x+y-t);
utx = @(x,y,t) -(cosh(x+y-t).^2-2)./cosh(x+y-t).^3;

phi = @(ep,r) 1./sqrt(1+(ep^2*r.^2)); % Inverse MQ-RBF

x = linspace(-L,L,n)'; [xm,ym] = meshgrid(x,x); data = [xm(:) ym(:)];
% Determind the position of boundary and interior points
Ibx = find(data(:,1)==L | data(:,1)==-L); 
Ibyp = find(data(:,2)==L); Ibym = find(data(:,2)==-L); 
Iby = [Ibyp(2:end-1);Ibym(2:end-1)]; Ib = [Ibx;Iby];
bdata = data; bdata(Ib,1)=2*L; Id = find(bdata(:,1)<L); 
Nd = length(data(Id,1)); Nb = length(data(Ib,1)); 
% Produce the fictitious points by linear mapping
linmap = @(x,y,x1,x2,y1,y2) [(y2-y1)*(x-x1)/(x2-x1) + y1  (y2-y1)*(y-x1)/(x2-x1) + y1];
dataF = linmap(data(Ib,1),data(Ib,2),x(2),x(n-1),-L,L);
data = [data; dataF]; N = length(data(:,1)); % RBF nodes
If = Nd+Nb+1:Nd+2*Nb;

%_________________% Figure of node distribution____________________________
figure(1); 
plot(data(If,1),data(If,2),'ok','MarkerFaceColor','g','MarkerSize',5);hold on;
plot(data(Ib,1),data(Ib,2),'or','MarkerFaceColor','m','MarkerSize',5);hold on;
plot(data(Id,1),data(Id,2),'ob','MarkerFaceColor','b','MarkerSize',4);hold on;
xcorner = [-L -L; -L L; L L; L -L; -L -L]; 
plot(xcorner(:,1),xcorner(:,2),'-k','linewidth',2); hold on 
set(gca,'Fontsize',15)
xlabel('x'); ylabel('y'); axis square;
xlim([-L-0.5, L+0.5]); ylim([-L-0.5, L+0.5]);
%__________________________________________________________________________

if nargin < 4
  h = min(pdist(data)); % Minimum distance of the node points
  ep = 0.1/h; % Shape parameter
end

% Distance and interpolation and differential matrices
dx = bsxfun(@minus,data(:,1),data(:,1).');
dy = bsxfun(@minus,data(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); A = phi(ep,r);
D1x = (-ep^2*dx.*A.^3)/A; D1y = (-ep^2*dy.*A.^3)/A; D1 = D1x + D1y;
D4 = (24*ep^4*A.^5 - 120*ep^6*r.^2.*A.^7 + 105*ep^8*r.^4.*A.^9)/A;
% Patitioning of boundary differentiation matrices
Bf = [D1x(Ibx,If); D1y(Iby,If)];
Bd = [D1x(Ibx,Id); D1y(Iby,Id)];
Bb = [D1x(Ibx,Ib); D1y(Iby,Ib)]; 
% Patitioning of differentiation matrices
D1d = D1(Id,Id) - (D1(Id,If)/Bf)*Bd;
D1bf = [D1(Id,Ib) - (D1(Id,If)/Bf)*Bb D1(Id,If)/Bf];
D4d = (D4(Id,Id) - (D4(Id,If)/Bf)*Bd);
D4bf = [D4(Id,Ib) - (D4(Id,If)/Bf)*Bb  D4(Id,If)/Bf];
% Initial condition
S0 = uini(data(Id,1),data(Id,2)); opt = odeset('RelTol',1e-8);
% Solve the ODE for the approximate solution S
fun = @(t,S) odefun(t,S,data,D1d,D4d,D1bf,D4bf,Id,Ib,ub,ux,ut,utx,gu);
[t,S] = ode15s(fun,[0 T],S0,opt);

%_______ Figures of approximated solutions_____________________
X=reshape(data(Id,1),n-2,n-2); Y=reshape(data(Id,2),n-2,n-2);
figure(2);
for i=1:length(t)
   mesh(X,Y,reshape(S(i,:)',n-2,n-2)); 
   set(gca,'Fontsize',15);
   xlabel('x'); ylabel('y'); zlabel('s(x,y,t)');
   zlim([0, 1.5]); view(50,40);
   pause(.01)
end
%________________________________________________________________
end

function Sprime = odefun(t,S,data,D1d,D4d,D1bf,D4bf,Id,Ib,ub,ux,ut,utx,gu)

Fx = [ub(data(Ib,1),data(Ib,2),t); ux(data(Ib,1),data(Ib,2),t)];
F4xt = [ut(data(Ib,1),data(Ib,2),t); utx(data(Ib,1),data(Ib,2),t)];
GuDx = gu(S).*(D1d*S + D1bf *Fx); I = eye(length(Id));
Sprime = (I + D4d)\(GuDx - D4bf*F4xt);

end

