%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve 2-D Rosenau equation on irregular domain with RBF
% collocation and Fictitious point method.
% u_t + \Delta^2{u} = \nabla g(u) , g(u) = u^2 + u^3.
% u(x,y,0) = sech(x + y)
% u(x,y,t) = f(x,y,t),    x \in \partial\Omega ,
% ux(x,y,t) = Df(x,y,t),  x \in \partial\Omega ,
% where f(x,y,t) = sech(x + y - t) 
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t,S] = FP_RBF_2DS(n,T,ep)
% n : The radially number of node points 
% T : Final time
% ep: Shape parameter. 
% For the case "FP_RBF_2DS(n,T)", the shape paramet is
% ep = 0.1/h; 

gu = @(u) 2*u + 3*u.^2; % Non-linear function g_u
uini = @(x,y) sech(x+y); % Initial condition
ub = @(x,y,t) sech(x+y-t); % Boundary condition
ux = @(x,y,t) -sech(x+y-t).*tanh(x+y-t);
ut = @(x,y,t) sech(x+y-t).*tanh(x+y-t);
utx = @(x,y,t) -(cosh(x+y-t).^2-2)./cosh(x+y-t).^3;

phi = @(ep,r) 1./sqrt(1+(ep^2*r.^2)); % Inverse MQ-RBF
rb = @(theta) (1+0.06*(sin(6*theta)+sin(3*theta))); % Domain boundary
% The normalized components of the normal
dr = @(theta) 0.06*(6*cos(6*theta)+3*cos(3*theta));
nx = @(theta)  dr(theta).*sin(theta)+rb(theta).*cos(theta); 
ny = @(theta) -dr(theta).*cos(theta)+rb(theta).*sin(theta); 
%----------------------------------------------------
% Interior node points 
r = linspace(0,1,n); dataI = [0 0]; dr=r(2)-r(1);
for i = 1:n-2
    th = [0: 2*pi/(7*i): 2*pi]'; th(end) = [];
    dataIi = [rb(th)*r(i+1).*cos(th) rb(th)*r(i+1).*sin(th)];
    dataI = [dataI; dataIi];
end
Nd = length(dataI(:,1));

% Boundary and Fictitious points
dataB = [rb(th).*cos(th)  rb(th).*sin(th)]; Nb = length(dataB(:,1));
dataF = [(1+dr)*rb(th).*cos(th)  (1+dr)*rb(th).*sin(th)];

data = [dataI; dataB; dataF]; N = length(data(:,1));
Id = 1:Nd; Ib = Nd+1:Nd+Nb; If = Nd+Nb+1:Nd+2*Nb;

if nargin < 3
  h = min(pdist(data)); % Minimum distance of the node points
  ep = 0.1/h; % Shape parameter
end
% The normalized components at the boundary points
nex = nx(th); ney = ny(th);
nr = hypot(nex,ney);
nu = [nex./nr   ney./nr];
% % ------ figure of normal vector at the boundary --------
% figure(1);
% plot(dataB(:,1),dataB(:,2),'b-'); hold on
% for k=1:Nb
%  plot(dataB(k,1)+[0 nu(k,1)],dataB(k,2)+[0 nu(k,2)],'k-')
% end
% %--------------------------------------------------------

%----------------------------------------------------
% Plot domain, interior nodes, boundary nodes and fictitious points 
thf = 0:2*pi/120:2*pi;
figure(2);clf;
plot(rb(thf).*cos(thf),rb(thf).*sin(thf),'k-','linewidth',2);hold on
plot(dataI(:,1),dataI(:,2),'ob','MarkerFaceColor','b','MarkerSize',3);hold on
plot(dataB(:,1),dataB(:,2),'om','MarkerFaceColor','m','MarkerSize',5);hold on
plot(dataF(:,1),dataF(:,2),'ok','MarkerFaceColor','g','MarkerSize',5);
set(gca,'Fontsize',15);
xlim([-1.2 1.3]); ylim([-1.3 1.2]);
xlabel('x'); ylabel('y'); axis square;
%--------------------------------------------------------

% Distance and interpolation and differential matrices
dx = bsxfun(@minus,data(:,1),data(:,1).');
dy = bsxfun(@minus,data(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); A = phi(ep,r);
D1x = (-ep^2*dx.*A.^3)/A; D1y = (-ep^2*dy.*A.^3)/A; D1 = D1x + D1y;
D4 = (24*ep^4*A.^5 - 120*ep^6*r.^2.*A.^7 + 105*ep^8*r.^4.*A.^9)/A;
% Patitioning of boundary of differentiation matrices
Bf = diag(nu(:,1))*D1x(Ib,If) + diag(nu(:,2))*D1y(Ib,If);
Bd = diag(nu(:,1))*D1x(Ib,Id) + diag(nu(:,2))*D1y(Ib,Id);
Bb = diag(nu(:,1))*D1x(Ib,Ib) + diag(nu(:,2))*D1y(Ib,Ib); 

% Patitioning of differentiation matrices
D1d = D1(Id,Id) - (D1(Id,If)/Bf)*Bd;
D1bf = [D1(Id,Ib) - (D1(Id,If)/Bf)*Bb D1(Id,If)/Bf];
D4d = (D4(Id,Id) - (D4(Id,If)/Bf)*Bd);
D4bf = [D4(Id,Ib) - (D4(Id,If)/Bf)*Bb  D4(Id,If)/Bf];

S0 = uini(dataI(:,1),dataI(:,2)); % initial condition
opt = odeset('RelTol',1e-8);
% Solve the ODE to approximate solution S
fun = @(t,S) odefun(t,S,dataB,D1d,D4d,D1bf,D4bf,Id,nu,ub,ux,ut,utx,gu);
[t,S] = ode15s(fun,[0 T],S0,opt);

% ---------------------------------------------------------------------
% Producing of the uniform mesh points to plot the numerical solution
r = 0:.05:.95; th = 0: 2*pi/(60): 2*pi;
[re,te] = meshgrid(r,th);
[x,y] = pol2cart(te,diag(rb(th))*re); eval= [x(:) y(:)];
% Rectangular projection from data to eval
dx = bsxfun(@minus,eval(:,1),dataI(:,1).');
dy = bsxfun(@minus,eval(:,2),dataI(:,2).');
Re = phi(ep,bsxfun(@hypot,dx,dy)); Re = Re/A(Id,Id); 
figure(3);
for i=1:length(t)
Se = reshape(Re*S(i,:)',length(th),length(r));
mesh(x,y,Se);
xlim([-1.2 1.2]); ylim([-1.2 1.2]); zlim([0 1.1]);
xlabel('x'); ylabel('y'); zlabel('s(x,y,t)');
set(gca,'Fontsize',15); view(30,32);pause(.05);
end
% -----------------------------------------------------------------------
end

function Sprime = odefun(t,S,dataB,D1d,D4d,D1bf,D4bf,Id,nu,ub,ux,ut,utx,gu)

Fx = [ub(dataB(:,1),dataB(:,2),t);...
(nu(:,1).*ux(dataB(:,1),dataB(:,2),t)+ nu(:,2).*ux(dataB(:,1),dataB(:,2),t))];
F4xt = [ut(dataB(:,1),dataB(:,2),t);...
nu(:,1).*utx(dataB(:,1),dataB(:,2),t)+nu(:,2).*utx(dataB(:,1),dataB(:,2),t)];
GuDx = gu(S).*(D1d*S + D1bf *Fx);
I = eye(length(Id));
Sprime = (I + D4d)\(GuDx - D4bf*F4xt);

end



