% Solve 1-D Rosenau equation on [-L,L] with RBF
% collocation method with fictitious points.
%
% Equation
% u_t + 0.5*u_xxxxt = g(u)_x
% 
% with boundary conditions
% u(-L,t) = sech(-L-t)
% u(+L,t) = sech(+L-t)
% u_x(-L,t) = sech(t+L)tanh(t+L)
% u_x(+L,t) = sech(t-L)tanh(t-L)
%
% Initial condition: u(x,t) = sech(x-t)
% g(u) = 10u^3 - 12u^5 - 3u/2
% g(u)_x = (-3/2)u_x - 60u^4u_x + 30u^2u_x
%
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.

function [S,x,t] = FP_RBF(N,L,T,ep)
% N : The number of node points
% L : [-L,L] is the domain
% T : Final time
% ep: Shape parameter. 
% For the case "fictitious(N,L,T)", the shape paramet is
% ep = 0.08/min(abs(diff(x)));

% Exact solution and derivatives for test case
u = @(x,t) sech(x-t);
ux = @(x,t) sech(t-x).*tanh(t-x);
ut = @(x,t) -sech(t-x).*tanh(t-x);
uxt = @(x,t) sech(t - x).^3 - sech(t - x).*tanh(t - x).^2;

% Generate nodes x. Map x to [-L,L] such that x(2) = -L, x(N-1) = L
% and x(1), x(N) are the left and right fictitious points respectively.
x = linspace(-L,L,N);
linmap = @(x,x1,x2,y1,y2) (y2-y1)*(x-x1)/(x2-x1) + y1;
x = linmap(x,x(2),x(N-1),-L,L); x = x(:);
if nargin < 4
 ep = 0.08/min(abs(diff(x))); % Shape parameter
end
% Differentiation matrices for inverse multiquadric RBF
phi = @(ep,r) 1./sqrt(1+(ep*r).^2);
dx = bsxfun(@minus,x,x.'); A = phi(ep,dx);
Dx = (-ep^2*dx.*A.^3)/A; % 1st Derivative matrix
D4x = (3*ep^4*(3-24*(ep*dx).^2+8*(ep*dx).^4).*A.^9)/A; % 4th Derivative
Bf = Dx([2 N-1],[1 N]); Bd = Dx([2 N-1],3:N-2); Bb = Dx([2 N-1],[2,N-1]);
% Modify differentiation matrices
Dxd = Dx(3:N-2,3:N-2) - (Dx(3:N-2,[1 N])/Bf)*Bd;
Dxb = [Dx(3:N-2,[2 N-1]) - (Dx(3:N-2,[1 N])/Bf)*Bb Dx(3:N-2,[1 N])/Bf];
D4xd = D4x(3:N-2,3:N-2) - (D4x(3:N-2,[1 N])/Bf)*Bd;
D4xb = [D4x(3:N-2,[2 N-1]) - (D4x(3:N-2,[1 N])/Bf)*Bb D4x(3:N-2,[1 N])/Bf];
% Initial condition
S0 = u(x(3:N-2),0); opt = odeset('RelTol',1e-10);
% Solve the ODE for the approximate solution S
fun = @(t,S) odefun(t,S,x,N,Dxd,Dxb,D4xd,D4xb,u,ux,ut,uxt);
[t,S] = ode15s(fun,[0 T],S0,opt);
% Plot the solution for all times
figure(1),clf,plot(x(3:N-2),S)
% Plot the error at the final time
E = abs(S(end,:)-u(x(3:N-2),T)');
figure(1),clf,plot(x(3:N-2),E)
% figure(3);
% for i=1:length(t)
% 	subplot(2,1,1)
% 	plot(x(3:N-2),S(i,:),'o',x(3:N-2),u(x(3:N-2),t(i)),'--r');
% 	xlabel('x'); ylabel('s(x,t)'); xlim([-L L]); ylim([0 1])
% 	drawnow;
% 	subplot(2,1,2)
% 	semilogy(t(i),max(E),'*b');
% 	xlabel('t'); ylabel('error'); ylim([1e-6 1e-5])
% 	set(gca,'ytick',10.^(-7:0)); hold on;
% end
function Sprime = odefun(t,S,x,N,Dxd,Dxb,D4xd,D4xb,u,ux,ut,uxt)
Fx = [u(x([2 N-1]),t); ux(x([2 N-1]),t)];
F4xt = [ut(x([2 N-1]),t); uxt(x([2 N-1]),t)];
Gu = diag(-1.5 - 60*S.^4 + 30*S.^2);
Sprime = (eye(N-4) + 0.5*D4xd)\(Gu*Dxd*S + Gu*Dxb*Fx - 0.5*D4xb*F4xt);
