% Solve 1-D Rosenau equation on [-L,L] with 
% Pseudo-Spectral Resampling (RS-PS) method
%
% Equation
% u_t + 0.5*u_xxxxt = g(u)_x
% 
% with boundary conditions
% u(-L,t) = sech(-L-t)
% u(+L,t) = sech(+L-t)
% u_x(-L,t) = sech(t+L)tanh(t+L)
% u_x(+L,t) = sech(t-L)tanh(t-L)
%
% Initial condition: u(x,t) = sech(x-t)
% g(u) = 10u^3 - 12u^5 - 3u/2
% g(u)_x = (-3/2)u_x - 60u^4u_x + 30u^2u_x

% REMARK:
% Some functions of Chebfun package is required
% Download Chebfun and add it to the path
% http://www.chebfun.org/

function [S,x,t] = RS_PS(N,L,T)
% N : The number of the cluster node points
% L : [-L,L] is the domain
% T : Final time

% Exact solution and derivatives for test case
u = @(x,t) sech(x-t);
ux = @(x,t) sech(t-x).*tanh(t-x);

x = L*chebpts(N); % Chebyshev points
% (N-4) auxiliary interior points
xt = L*chebpts(N-4);
% Differentiation matrices
D = (1/L)*diffmat(N); D4 = (1/L)^4*diffmat(N,4);
% First derivative matrix at x = -L and x = L
Bt = [D(1,:); D(N,:)];
% Rectangular projection from x to xt
R = barymat(xt,x);
% Initial condition
S0 = u(x,0);
M = [ R*(eye(N)+ 0.5*D4); zeros(4,N) ];
opt = odeset('mass',M,'masssing','yes','RelTol',1e-9);
[t,S] = ode15s(@(t,S) daefun(t,S,x,N,R,D,Bt,u,ux),[0 T],S0,opt);
% Plot the solution for all times
figure(1),clf,plot(x,S)
% Plot the error at the final time
E = abs(S(end,:)-u(x,T)');
figure(2),clf,plot(x,E)
% figure(3);
% for i=1:length(t)
% 	subplot(2,1,1)
% 	plot(x,S(i,:),'o',x,u(x,t(i)),'--r');
% 	xlabel('x'); ylabel('s(x,t)'); xlim([-L L]); ylim([0 1])
% 	drawnow;
% 	subplot(2,1,2)
% 	semilogy(t(i),max(E),'*b');
% 	xlabel('t'); ylabel('error'); ylim([1e-6 1e-5])
% 	set(gca,'ytick',10.^(-7:0)); hold on;
% end
function Sprime = daefun(t,S,x,N,R,D,Bt,u,ux)
F = [zeros(N-4,1); u(x([1 N]),t); ux(x([1 N]),t)];
Gu = diag(- 60*(R*S).^4 + 30*(R*S).^2 - 1.5);
I = eye(N);
Sprime = [Gu*(R*D); [I(1,:); I(N,:)]; Bt]*S - F;
