% Solve 1-D Rosenau equation on [-L,L] with RBF
% collocation method with Resampling RBF method.
%
% Equation
% u_t + 0.5*u_xxxxt = g(u)_x
% 
% with boundary conditions
% u(-L,t) = sech(-L-t)
% u(+L,t) = sech(+L-t)
% u_x(-L,t) = sech(t+L)tanh(t+L)
% u_x(+L,t) = sech(t-L)tanh(t-L)
%
% Initial condition: u(x,t) = sech(x-t)
% g(u) = 10u^3 - 12u^5 - 3u/2
% g(u)_x = (-3/2)u_x - 60u^4u_x + 30u^2u_x
%
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.

function [S,x,t] = RS_RBF(N,L,T,ep)
% N : The number of node points
% L : [-L,L] is the domain
% T : Final time
% ep: Shape parameter. 
% For the case "RS_RBF(N,L,T)", the shape paramet is
% ep = 0.08/min(abs(diff(x)));

% Exact solution and derivatives for test case
u = @(x,t) sech(x-t);
ux = @(x,t) sech(t-x).*tanh(t-x);
% Generate N uniform RBF nodes with boundary pts last
x = linspace(-L,L,N).';
x = [x(2:end-1); x([1 end])];
% Generate N-4 uniform auxiliary interior points
xt = linspace(-L,L,N-2).'; xt([1 end]) = [];
% Differentiation matrices for inverse multiquadric RBF
phi = @(ep,r) 1./sqrt(1+(ep*r).^2);
if nargin < 4
 ep = 0.08/min(abs(diff(x))); % Shape parameter
end
r = bsxfun(@minus,x,x.'); A = phi(ep,r);
% First derivative matrix at x = -L and x = L
r = bsxfun(@minus,x([N-1 N]),x.');
Bt = (-ep^2*r.*phi(ep,r).^3)/A;
% Rectangular projection from x to xt
r = bsxfun(@minus,xt,x.'); R = phi(ep,r);
PRx = (-ep^2*r.*R.^3)/A; PR = R/A;
PR4x = (3*ep^4*(3-24*(ep*r).^2+8*(ep*r).^4).*R.^9)/A;
% Initial condition
S0 = u(x,0);
M = [(PR + 0.5*PR4x); zeros(4,N)];
opt = odeset('mass',M,'masssing','yes','RelTol',1e-9);
[t,S] = ode15s(@(t,S) daefun(t,S,x,N,PR,PRx,Bt,u,ux),[0 T],S0,opt);
% Plot the solution for all times
ind = [N-1 1:N-2 N];
figure(1),clf,plot(x(ind),S(:,ind))
% Plot the error at the final time
E = abs(S(end,:)-u(x,T)');
figure(2),clf,plot(x(ind),E(ind))
% figure(3);
% for i=1:length(t)
% 	subplot(2,1,1)
% 	plot(x(ind),S(i,ind),'o',x(ind),u(x(ind),t(i)),'--r');
% 	xlabel('x'); ylabel('s(x,t)'); xlim([-L L]); ylim([0 1])
% 	drawnow;
% 	subplot(2,1,2)
% 	semilogy(t(i),max(E),'*b');
% 	xlabel('t'); ylabel('error'); ylim([1e-6 1e-5])
% 	set(gca,'ytick',10.^(-7:0)); hold on;
% end
function Sprime = daefun(t,S,x,N,PR,PRx,Bt,u,ux)
F = [zeros(N-4,1); u(x([N-1 N]),t); ux(x([N-1 N]),t)];
Gu = diag(- 60*(PR*S).^4 + 30*(PR*S).^2 - 1.5);
Sprime = [Gu*PRx; [zeros(2,N-2) eye(2,2)]; Bt]*S - F;



