%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve 2-D Rosenau equation on [-L,L]x[-L,L] with RBF
% collocation and Resampling RBF method.
% u_t + \Delta^2{u} = \nabla g(u) , g(u) = u^2 + u^3.
% u(x,y,0) = sech(x + y)
% u(x,y,t) = f(x,y,t),    x \in \partial\Omega ,
% ux(x,y,t) = Df(x,y,t),  x \in \partial\Omega ,
% where f(x,y,t) = sech(x + y - t) 
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t,S] = RS_RBF_2D(n,L,T,ep)
% n : The number of node points in each direction
% L : [-L,L]x[-L,L] is the domain
% T : Final time
% ep: Shape parameter. 
% For the case "RS_RBF_2D(n,L,T)", the shape paramet is
% ep = 0.12/h;

gu = @(u) 2*u + 3*u.^2; % Non-linear function g_u
uini = @(x,y) sech(x+y); % Initial condition
ub = @(x,y,t) sech(x+y-t); % Boundary condition
uxb = @(x,y,t) -sech(x+y-t).*tanh(x+y-t);

phi = @(ep,r) 1./sqrt(1+(ep^2*r.^2)); %Inverse MQ-RBF

x = linspace(-L,L,n)'; x([1 end]) = [];  % Interior nodes
[xm,ym] = meshgrid(x,x); dataI = [xm(:) ym(:)]; 
% Produce the boundary nodes
xb = [-L  L]; yb = linspace(-L,L,n-2);
[xb1,yb1] = meshgrid(xb,yb); [xb2,yb2] = meshgrid(yb(2:end-1),xb);
dataB = [xb1(:),yb1(:); xb2(:),yb2(:)]; 
data = [dataI;dataB]; % Interior and boundary nodes 
Nd = length(dataI(:,1)); Nb = length(dataB(:,1)); 
Id=1:Nd; Ib=Nd+1:Nd+Nb;
% Determind the boundary 
Ibx = find(data(:,1)==L | data(:,1)==-L); 
Ibyp = find( data(:,2)==L & data(:,1)~=L & data(:,1)~=-L); 
Ibym = find(data(:,2)==-L & data(:,1)~=L & data(:,1)~=-L);
Iby = [Ibyp;Ibym]; Ib = [Ibx;Iby];
%  auxiliary  points
xt = linspace(-L,L,n-2).'; xt([1 end]) = [];
[xtm,ytm] = meshgrid(xt,xt); tdata = [xtm(:) ytm(:)];  

%_________________% Figure of node distribution____________________________
figure(1); 
plot(data(Id,1),data(Id,2),'ob','MarkerFaceColor','b','MarkerSize',2); hold on
plot(tdata(:,1),tdata(:,2),'*k','MarkerSize',6); hold on
plot(data(Ib,1),data(Ib,2),'om','MarkerFaceColor','m','MarkerSize',5);
xcorner = [-L -L;-L L;L L;L -L;-L -L]; 
plot(xcorner(:,1),xcorner(:,2),'-k','linewidth',2); hold off
set(gca,'Fontsize',15); xlim([-L-.5, L+.5]); ylim([-L-.5, L+.5]);
xlabel('x'); ylabel('y'); axis square;
%__________________________________________________________________________

if nargin < 4
  h = min(pdist(data)); % Minimum distance of the node points
  ep = 0.12/h; % Shape parameter
end
% Use Inverse Multiquadric RBF
phi = @(ep,r) 1./sqrt(1+(ep*r).^2); 
% Distance and interpolation Matrix
dx = bsxfun(@minus,data(:,1),data(:,1).');
dy = bsxfun(@minus,data(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); A = phi(ep,r);
% First partial derivative matrix at boundary points
D1bx = (-ep^2*dx.*phi(ep,r).^3)/A; 
D1by = (-ep^2*dy.*phi(ep,r).^3)/A; 
Dbx = D1bx(Ibx,:); Dby = D1by(Iby,:); Db = [Dbx;Dby];
% Rectangular projection from data to tdata
dx = bsxfun(@minus,tdata(:,1),data(:,1).');
dy = bsxfun(@minus,tdata(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); R = phi(ep,r);
R1 = (-ep^2*(dx + dy).*R.^3)/A;
R4 = (24*ep^4*R.^5 - 120*ep^6*r.^2.*R.^7 + 105*ep^8*r.^4.*R.^9)/A; R = R/A;

M = [(R + R4); zeros(2*Nb,Nd+Nb)]; % Mass matrix 
S0 = uini(data(:,1),data(:,2)); % initial condition
opt = odeset('Mass',M,'masssing','yes','RelTol',1e-7);
[t,S] = ode15s(@daefun,[0 T],S0,opt);

%_______ Figures of approximated solutions_____________________
X=reshape(data(Id,1),n-2,n-2); Y=reshape(data(Id,2),n-2,n-2);
figure(2);
for i=1:length(t)
   mesh(X,Y,reshape(S(i,1:(n-2)^2)',n-2,n-2)); 
   set(gca,'Fontsize',15);
   xlabel('x'); ylabel('y'); zlabel('s(x,y,t)');
   zlim([0, 1.5]); view(50,40);
   pause(.01)
end
%____________________________________________________________

function Sprime = daefun(t,S)
    
   F =[zeros(Nd-Nb,1); ub(data(Ib,1),data(Ib,2),t); uxb(data(Ib,1),data(Ib,2),t)];
   I = eye(Nd+Nb,Nd+Nb); 
   Sprime = [diag(gu(R*S))*R1; I(Ib,:); Db]*S - F;
end 
end