%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve 2-D Rosenau equation on irregular domain with RBF
% collocation and Resampling RBF method.
% u_t + \Delta^2{u} = \nabla g(u) , g(u) = u^2 + u^3.
% u(x,y,0) = sech(x + y)
% u(x,y,t) = f(x,y,t),    x \in \partial\Omega ,
% ux(x,y,t) = Df(x,y,t),  x \in \partial\Omega ,
% where f(x,y,t) = sech(x + y - t) 
% Copyright (C) 2015 Ali Safdari, Elisabeth Larsson and Alfa Heryudono
% This MATLAB code is released to support reproducible research for numerical 
% results in the manuscript:
% Radial basis function methods for the Rosenau equation and
% other higher order PDEs
% Status: Submitted
% Ali Safdari-Vaighani, Elisabeth Larsson, Alfa Heryudono.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t,S] = RS_RBF_2DS(n,T,ep)
% n : The radially number of node points 
% T : Final time
% ep: Shape parameter. 
% For the case "RS_RBF_2DS(n,T)", the shape paramet is
% ep = 0.1/h; 

gu = @(u) 2*u + 3*u.^2; % Non-linear function g_u
uini = @(x,y) sech(x+y); % Initial condition
ub = @(x,y,t) sech(x+y-t); % Boundary conditions
ubx = @(x,y,t) -sech(x+y-t).*tanh(x+y-t);

phi = @(ep,r) 1./sqrt(1+(ep^2*r.^2)); % Inverse MQ-RBF
rb = @(theta) (1+0.06*(sin(6*theta)+sin(3*theta))); % Domain boundary
% The normalized components of the normal
dr = @(theta) 0.06*(6*cos(6*theta)+3*cos(3*theta));
nx = @(theta)  dr(theta).*sin(theta)+rb(theta).*cos(theta); 
ny = @(theta) -dr(theta).*cos(theta)+rb(theta).*sin(theta); 
%----------------------------------------------------
% Interior node points 
r = linspace(0,1,n); dataI = [0 0];
for i = 1:n-2
    th = [0: 2*pi/(7*i): 2*pi]'; th(end) = [];
    dataIi = [rb(th)*r(i+1).*cos(th) rb(th)*r(i+1).*sin(th)];
    dataI = [dataI; dataIi];
end 
% Boundary node points 
dataB = [rb(th).*cos(th)  rb(th).*sin(th)]; 
data = [dataI; dataB]; 
N = length(data(:,1)); Nd = length(dataI(:,1)); Nb = length(dataB(:,1));

if nargin < 3
  h = min(pdist(data)); % Minimum distance of the node points
  ep = 0.1/h; % Shape parameter
end
% The normalized components at the boundary points
nex = nx(th); ney = ny(th);
nr = hypot(nex,ney);
nu = [nex./nr   ney./nr];
% % ------ figure of normal vector at the boundary --------
% figure(1);
% plot(dataB(:,1),dataB(:,2),'b-'); hold on
% for k=1:Nb
%  plot(dataB(k,1)+[0 nu(k,1)],dataB(k,2)+[0 nu(k,2)],'k-')
% end
% %--------------------------------------------------------
% Auxiliary node points  
r = linspace(0,1,n-1);
tdata = [0 0];
for i = 1:n-3
    th = [0: 2*pi/(7*i): 2*pi]'; th(end) = [];
    tdataI=[rb(th)*r(i+1).*cos(th) rb(th)*r(i+1).*sin(th)];
    tdata = [tdata; tdataI];
end
%----------------------------------------------------
% Plot domain, interior nodes, boundary nodes and uxiliary points 
thf = 0:2*pi/120:2*pi;
figure(2);clf;
plot(rb(thf).*cos(thf),rb(thf).*sin(thf),'k-','linewidth',2);hold on
plot(dataI(:,1),dataI(:,2),'ob','MarkerFaceColor','b','MarkerSize',3);hold on
plot(dataB(:,1),dataB(:,2),'om','MarkerFaceColor','m','MarkerSize',5);hold on
plot(tdata(:,1),tdata(:,2),'*k','MarkerSize',6);
set(gca,'Fontsize',15);
xlim([-1.2 1.3]); ylim([-1.3 1.2]);
xlabel('x'); ylabel('y'); axis square;
%--------------------------------------------------------
% Distance and interpolation Matrix
dx = bsxfun(@minus,data(:,1),data(:,1).');
dy = bsxfun(@minus,data(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); A = phi(ep,r);
dx = bsxfun(@minus,dataB(:,1),data(:,1).');
dy = bsxfun(@minus,dataB(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); %A = phi(ep,r);
% First partial derivative matrix at boundary points
D1x = (-ep^2*dx.*phi(ep,r).^3)/A; 
D1y = (-ep^2*dy.*phi(ep,r).^3)/A; 
Db = (diag(nu(:,1))*D1x + diag(nu(:,2))*D1y);

% Rectangular projection from data to tdata
dx = bsxfun(@minus,tdata(:,1),data(:,1).');
dy = bsxfun(@minus,tdata(:,2),data(:,2).');
r = bsxfun(@hypot,dx,dy); R = phi(ep,r);
R1 = (-ep^2*(dx + dy).*R.^3)/A;
R4 = (24*ep^4*R.^5 - 120*ep^6*r.^2.*R.^7 + 105*ep^8*r.^4.*R.^9)/A; R = R/A;
M = [(R + R4); zeros(2*Nb,N)]; % Mass matrix 
S0 = uini(data(:,1),data(:,2)); % initial condition
% Solve the DAE to approximate solution S
opt = odeset('mass',M,'masssing','yes','RelTol',1e-8);
[t,S] = ode15s(@daefun,[0 T],S0,opt);

% ---------------------------------------------------------------------
% Producing of the uniform mesh points to plot the numerical solution
r = 0:.05:.95; th = 0: 2*pi/(60): 2*pi;
[re,te] = meshgrid(r,th);
[x,y] = pol2cart(te,diag(rb(th))*re); eval= [x(:) y(:)];
% Rectangular projection from data to eval
dx = bsxfun(@minus,eval(:,1),data(:,1).');
dy = bsxfun(@minus,eval(:,2),data(:,2).');
Re = phi(ep,bsxfun(@hypot,dx,dy)); Re = Re/A; 
figure(3);
for i=1:length(t)
Se = reshape(Re*S(i,:)',length(th),length(r));
mesh(x,y,Se);
xlim([-1.2 1.2]); ylim([-1.2 1.2]); zlim([0 1.1]);
xlabel('x'); ylabel('y'); zlabel('s(x,y,t)');
set(gca,'Fontsize',15); view(30,32);pause(.05);
end
% -----------------------------------------------------------------------

function Sprime = daefun(t,S)
F = [zeros(N-2*Nb,1); ub(dataB(:,1),dataB(:,2),t);
   (nu(:,1).*ubx(dataB(:,1),dataB(:,2),t)+ nu(:,2).*ubx(dataB(:,1),dataB(:,2),t))];
I = eye(N); 
Sprime = [diag(gu(R*S))*R1; I(Nd+1:end,:); Db]*S - F;
end 
end

